<?php
/*******************************************
 @File		utep_dhcp.php
 @Comment	helper function for Authenticat0r
 			will auth against DHCP authenticator service @UTEP
 @written 	8/27/05, 23:55
 @Author 	derDoc
 @email 	derDoc@gmx.de

 @Changelog
-> v1.0: 8/27/05, 23:55
	-basic auth working
	-gets statically reloaded every 1800secs
	-displays ping to www.nmsu.edu
-> v1.2: 9/22/05, 14:45
	-moved everything into functions
	-changed code of OnlineChecker to raw http get
	-lill nicer page layout
	-added form for options
	-extensively customizable thru vars
-> v1.4: 9/24/05, 12:30
	-added $max_retries var
	-added <hr> before output from auth()
	-reniced output from auth() a little (more info)
	-added execution time
	-added Xping (times to ping)
	-made count option of ping work under windows (-n)
	-added ver to header
-> v2.0: 27.9.2k5, 23:30
	-final rewrite of code
	-moved dhcp auth() into 'utep_dhcp.php' and
	-moved wlan auth() into 'utep_wlan.php'
	-wrote 'online_script.php', which can be run as script (performance)
	-wrote 'online_page.php' does the whole page thingy with a nice form...
	-both will use approb. code to authenticate (dhcp || wlan)
	-moved all constants from vars to define(), so that funcs see them
	-isOpen() now smarter
	-begin() decides what code to include based on HTTP status code
	-added http_statcode, which only gets 13 chars
	-script checks conn and dies if open, good for running regularily with cron
	-script turnes output buffering on and thus supresses any output
	-auth() now exclusively uses SITE200OK for inserting into data (GOTO is only defined in online_page)
	-isOpen() first checks outside conn by opening plain socket to OUTSIDE_IP on OUTSIDE_PORT w/ TO=1
	-for DHCP this host is set to first hop after utep1.utep.edu (192.88.12.9) on MSDP (to not raise too much suspicion), because inside hosts are still reachable
	-for WLAN this can be set to an inside host (eg. rlab.cs), saves 0.01s
	-pinging does not work, because firstly on DHCP pings will go through, and sec.ly TO for ping is higher than 1
	-port 80 will be forwarded to 10.0.1.7, so this too will not work
-> v2.2: 29.9.2k5, 22:30
	-cleaned header slightly
	-added comments
	-line_delims
-> v2.2.1: 6.2.2k6, 11:50
	-added a global that holds the ip we authenticated (normally SITE200OK)
	-added code spot (byname)
-> v2.2.2: 9.2.2k6, 11:54
	-changed debug level of raw-output to >1 (INSANE)
	-changed debug level of all other output to 1 (ALL)
	-added output of raw_get and raw_read when DEBUG>1 

********************************************/


/* @Function	auth()
 * @Params		no
 * @Returns		no
 * authenticates against DHCP authentication service @ UTEP
*/
function auth(){
	global $authed_ip;
	$return_code = 255;
	$i = 0;

	if (DEBUG) echo "dhcp->auth: Authenticating:<br />".LINE_DELIM;
	while ( !isOpen() && $i<MAX_RETRIES ){
		if (DEBUG) echo "dhcp->auth: No connection to ".SITE200OK.", opening connection...<br />".LINE_DELIM;
		$fp = fsockopen(SITE200OK, 80, $errno, $errstr, 30);
		if ($fp){
			if (DEBUG) echo "dhcp->auth: Connection to ".SITE200OK." open, authenticating...<br />".LINE_DELIM;
	   		$out = "GET / HTTP/1.1\r\n";
   			$out .= "Host: ".SITE200OK."\r\n";
   			$out .= "Authorization: Basic ".base64_encode(USER.':'.PASS)."\r\n";
	   		$out .= "Connection: close\r\n\r\n";
   			
   			if (DEBUG > 1) echo "dhcp->auth: raw_get is<br><pre>$out</pre><br><hr>";
			fwrite($fp, $out);
   			if (DEBUG) echo "dhcp->auth: Authenticated..., reading:<br />".LINE_DELIM;
			while (!feof($fp))
    			$in .= fgets($fp, 128);
			fclose($fp);
			if (DEBUG > 1) echo "dhcp->auth: raw_read is<br><pre>$in</pre><br><hr>";
			
			if (empty($in))
				if (DEBUG) echo "dhcp->auth: Failed: nothing read!<br />".LINE_DELIM;
			else if(substr($in, 9, 1) != '2')
   				if (DEBUG) echo "dhcp->auth: Failed: Code " . (substr($in, 9, 3)) . "<br />".LINE_DELIM;
   			else
   				if (DEBUG) echo "dhcp->auth: OK: Code ".substr($in, 9, 3)."<br />".LINE_DELIM;
	   	}else{
   			if (DEBUG) echo "dhcp->auth: Open Socket failed: $errstr ($errno)<br />".LINE_DELIM;
   		}
		$i++;
	}
	if (!isOpen()){
		if (DEBUG) echo "dhcp->auth: Failed to authenticate!<br /><hr />".LINE_DELIM;
		$return_code = 1;
		
	}else{
		if (DEBUG) echo "dhcp->auth: Connection open!<br /><hr />".LINE_DELIM;
		$return_code = 0;
		$authed_ip = gethostbyname(SITE200OK);
	}
	
	return ($return_code);
}

?>
